/*
===============================================================================
 Name        : uart.h
 Author      : Eduardo Rafael Sepulveda de Ugarte
 Version     : 1.0
 Description : Header file for uart.c
 ===============================================================================
 */
#include "lpc_types.h"

#ifndef __UART_H 
#define __UART_H

#define IER_RBR		0x01
#define IER_THRE	0x02
#define IER_RLS		0x04

#define IIR_PEND	0x01
#define IIR_RLS		0x03
#define IIR_RDA		0x02
#define IIR_CTI		0x06
#define IIR_THRE	0x01

#define LSR_RDR		0x01
#define LSR_OE		0x02
#define LSR_PE		0x04
#define LSR_FE		0x08
#define LSR_BI		0x10
#define LSR_THRE	0x20
#define LSR_TEMT	0x40
#define LSR_RXFE	0x80

// Accepted Error baud rate value (in percent unit) 
#define UART_ACCEPTED_BAUDRATE_ERROR	(3)			    /*!< Acceptable UART baudrate error */

//#define UART_LCR_DLAB_EN		((uint8_t)(1<<7))    	/*!< UART Divisor Latches Access bit enable */
//#define UART_LCR_BITMASK		((uint8_t)(0xFF))		/*!< UART line control bit mask */

/*********************************************************************//**
 * Macro defines for Macro defines for UART Fractional divider register
 **********************************************************************/
#define UART_FDR_DIVADDVAL(n)	((uint32_t)(n&0x0F))		/**< Baud-rate generation pre-scaler divisor */
#define UART_FDR_MULVAL(n)		((uint32_t)((n<<4)&0xF0))	/**< Baud-rate pre-scaler multiplier value */
#define UART_FDR_BITMASK		((uint32_t)(0xFF))			/**< UART Fractional Divider register bit mask */

/*********************************************************************//**
 * Macro defines for Macro defines for UARTn Divisor Latch LSB register
 **********************************************************************/
#define UART_LOAD_DLL(div)	((div) & 0xFF)	/**< Macro for loading least significant halfs of divisors */
//#define UART_DLL_MASKBIT	((uint8_t)0xFF)	/*!< Divisor latch LSB bit mask */

/*********************************************************************//**
 * Macro defines for Macro defines for UARTn Divisor Latch MSB register
 **********************************************************************/
#define UART_LOAD_DLM(div)  (((div) >> 8) & 0xFF)	/**< Macro for loading most significant halfs of divisors */
//#define UART_DLM_MASKBIT	((uint8_t)0xFF)			/*!< Divisor latch MSB bit mask */

void UART0_IRQHandler( void );
void UART1_IRQHandler( void );
void UART2_IRQHandler( void );
void UART3_IRQHandler( void );
uint32_t UART_Init( uint8_t PortNum, uint32_t Baudrate );
uint32_t UART_Close( uint8_t PortNum);
uint32_t UART_Set_Divisors(uint8_t PortNum, uint32_t Baudrate);
void UART_Write( uint8_t PortNum, uint8_t send);
void UART_Send( uint8_t PortNum, char *BufferPtr);
void UART_Flush (uint8_t PortNum);

#endif /* end __UART_H */
/*****************************************************************************
**                            End Of File
******************************************************************************/
