/*
===============================================================================
 Name        : main.c
 Author      : Eduardo Rafael Sepulveda de Ugarte
 Version     : 1.0
 Description : Main function:
  	  	  	  	-Initialization of all modules.
  	  	  	  	-Tasks definitions.
  	  	  	  	-Throw all tasks.
===============================================================================
*/

//	LPC Library
#include "LPC17xx.h"

/* ------ FreeRTOS.org includes. -------------------*/
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "basic_io.h"
#include <cr_section_macros.h>

/* ------ RedLib. ----------------------------------*/
#include <stdio.h>

/* ------ Type definition files. -------------------*/
#include "lpc_types.h"
#include "config.h"

/* ------ PickMouse includes. ----------------------*/
#include "leds.h"
#include "gpio.h"
#include "uart.h"
#include "wifly.h"
#include "printf.h"
#include "adc.h"
#include "acel.h"

// Used as a loop counter to create a very crude delay. 
#define BIG_DELAY_LOOP_COUNT		( 0xfffff )
#define MAIN_DELAY_LOOP_COUNT		( 0xaf )
#define MID_DELAY_LOOP_COUNT		( 0xff )
#define DELAY_LOOP_COUNT1			( 0xfff )

#define DELAY_BOTTON				( 5 )
#define DELAY_BOTTON1				( 10 )
#define DELAY_BOTTON2				( 500 )

// Adc Channels
#define CHANNX	0
#define CHANNY	1
#define CHANNZ	2

/* Init Functions		*/
void IO_Init();
void ADC_Init();
void Config_Wifly ();

/* The task functions. 	*/
static void ReadButtontTask(void *pvParameters );
static void MotionTask(void *pvParameters);

bool enable = false;

/*-----------------------------------------------------------*/
int main( void )
{

	printf( "\n" );
	printf( "[main] Start...\n" );

	printf( "[main] Init UART3 DEBUG...\n" );
	Print_Flush	(UART_DBG);
	Print_Init	(UART_DBG, BR_DBG);
	Print_Flush	(UART_DBG);
	Print_Text	(UART_DBG, (char *) "\n\rUART Started.");

	printf( "\n[main] Init Leds...");
	led_init();
	led_off();
	led_on();
	led_off();

	printf( "\n[main] Init Buttons...");
	IO_Init();

	printf( "\n[main] Initializing ADC..." );
	ADC_Init();

	printf( "\n[main] Getting repose values...");
	ADC_GetReposeValues(CHANNX,CHANNY,CHANNZ);

	printf( "\n[main] Reseting values...");
	ADC_ResetVals();
	ADC_ResetDiffs();

 // Force the counter to be placed into memory
    volatile static int i = 0 ;

    //============================ WIFI ===========================================
    Config_Wifly();

    volatile unsigned long ul;

	printf("\nWaiting for Wifly module reboot...");
	int j=0;
	for (j=0; j<1; j++)
	{
		for( ul = 0; ul < MAIN_DELAY_LOOP_COUNT; ul++ )
		{
			printf( "." );
		}
		printf("\n");
	}

    enable = true;
    led_on();

	printf( "\n[Init] Creating Tasks ...");
//	xTaskCreate(	vTask1,		/* Pointer to the function that implements the task. */
//					"Task 1",	/* Text name for the task.  This is to facilitate debugging only. */
//					240,		/* Stack depth in words. */
//					NULL,		/* We are not using the task parameter. */
//					1,			/* This task will run at priority 1. */
//					NULL );		/* We are not using the task handle. */

	signed portBASE_TYPE return_value;

	return_value = xTaskCreate	( ReadButtontTask,
								( signed portCHAR * ) "ButtonTask",
								configMINIMAL_STACK_SIZE,
								NULL,
								1,
								NULL );
	if( return_value == pdPASS )
		printf( "\n[main] MotionTask Created!!!!");


	return_value = xTaskCreate	( MotionTask,
								( signed portCHAR * ) "MotionTask",
								configMINIMAL_STACK_SIZE,
								NULL,
								1,
								NULL );

	if( return_value == pdPASS )
		printf( "\n[main] MotionTask Created!!!!");

	printf( "\n[main] Starting Scheduler...");
	vTaskStartScheduler();


	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
	return 0;
}
/*-----------------------------------------------------------*/

static void ReadButtontTask(void *pvParameters)
{
	 uint32_t var1 = 0;
	 uint32_t var2 = 0;
	 uint32_t var3 = 0;
	 uint32_t var4 = 0;

	 uint32_t last_var1 = 0;
	 uint32_t last_var2 = 0;
	 uint32_t last_var3 = 0;
	 uint32_t last_var4 = 0;
	 volatile unsigned long ul;

	 vPrintString( "\n[ReadButtontTask] Start reading:" );
	 while(1)
	 {
		  var1 = GPIOGetValue(0, 2);          // read the button value
		  var2 = GPIOGetValue(0, 3);          // read the button value
		  var3 = GPIOGetValue(0, 21);         // read the button value
		  var4 = GPIOGetValue(2, 6);         // read the button value

		  if ( (var1 == 1) && (var1 != last_var1) )
		  {
			  vPrintString ( "\n [ReadButtontTask] X." );
		  	  Print_Text(UART_WF,"X");
		  }

		  if ( (var2 == 1) && (var2 != last_var2) )
		  {
			  vPrintString( "\n [ReadButtontTask] Y." );
			  Print_Text(UART_WF,"Y");
		  }

		  if ( (var3 == 1) && (var3 != last_var3) )
		  {
			  vPrintString( "\n [ReadButtontTask] Z." );
		  	  Print_Text(UART_WF,"Z");
		  }

		  if ( (var4 == 1) && (var4 != last_var4) )
		  {
			  vPrintString( "\n [ReadButtontTask] W." );
			  if (enable)
			  {
				  enable = false;
			  	  vPrintString( "\n [MovementsTask] Disabling" );
			  	  led_off();
			  }
			  else
			  {
				  enable = true;
				  vPrintString( "\n [MovementsTask] Enabling" );
				  led_on();
			  }

			  Print_Text(UART_WF,".");
			  //for( ul = 0; ul < BIG_DELAY_LOOP_COUNT; ul++ ){}
		  }

		  last_var1 = var1;
		  last_var2 = var2;
		  last_var3 = var3;
		  last_var4 = var4;

		  vTaskDelay (DELAY_BOTTON);

		  for( ul = 0; ul < MID_DELAY_LOOP_COUNT; ul++ ){}
	 }
}

//static void MotionTask()
static void MotionTask(void *pvParameters)
{
	vPrintString( "\n[MovementsTask] Start reading:\n" );
	int cont=0;
	volatile unsigned long ul;

	bool discartX = false;
	bool discartY = false;
	bool discartZ = false;

	int x_move = 0;
	int y_move = 0;

	while (1)
	{
		if (!enable)
		{
			//vPrintString( "\n [MovementsTask] DISABLED!:" );
			for( ul = 0; ul < BIG_DELAY_LOOP_COUNT; ul++ ){}
		}
		else
		{
			//vPrintString( "\n [MovementsTask]  ENABLED:" );
			ADC_GetAcel(CHANNX, CHANNY, CHANNZ, false, true);

			if (cont > 1)
			{
				ADC_CalculateDiffs();

				if (ADC_ExtreamsValuesFilter())
					ADC_CalculateDiffs();

				ADC_LookForNoise(&discartX,&discartY,&discartZ);

				ADC_DiscartNoiseData(discartX,discartY,discartZ);

				if ( discartX || discartY)// || discartZ )
					ADC_CalculateDiffs();
				else
				{
					x_move = ADC_LookFor_Start_Stop_X();
					y_move = ADC_LookFor_Start_Stop_Y();
				}


				//vPrintString( "\n");

				discartX = false;
				discartY = false;
				discartZ = false;

				//ADC_ResetDiscarts();
			}

			ADC_MoveValues();

			switch (x_move)
			{
			case 1:
				vPrintString( "______________> Sending Up   ");
				Print_Text(UART_WF,"A");
				break;
			case 2:
				vPrintString( "______________> Sending Up+  ");
				Print_Text(UART_WF,"B");
				break;
			case 3:
				vPrintString( "______________> Sending Up++ ");
				Print_Text(UART_WF,"C");
				break;
			case 4:
				vPrintString( "______________> Sending Up+++");
				Print_Text(UART_WF,"D");
				break;
			case -1:
				vPrintString( "______________> Sending Down   ");
				Print_Text(UART_WF,"F");
				break;
			case -2:
				vPrintString( "______________> Sending Down+  ");
				Print_Text(UART_WF,"G");
				break;
			case -3:
				vPrintString( "______________> Sending Down++ ");
				Print_Text(UART_WF,"H");
				break;
			case -4:
				vPrintString( "______________> Sending Down++");
				Print_Text(UART_WF,"I");
				break;
			}
			switch (y_move)
			{
			case 1:
				vPrintString( "______________> Sending Left   ");
				Print_Text(UART_WF,"K");
				break;
			case 2:
				vPrintString( "______________> Sending Left+  ");
				Print_Text(UART_WF,"L");
				break;
			case 3:
				vPrintString( "______________> Sending Left++ ");
				Print_Text(UART_WF,"M");
				break;
			case 4:
				vPrintString( "______________> Sending Left+++");
				Print_Text(UART_WF,"N");
				break;
			case -1:
				vPrintString( "______________> Sending Right   ");
				Print_Text(UART_WF,"P");
				break;
			case -2:
				vPrintString( "______________> Sending Right+  ");
				Print_Text(UART_WF,"Q");
				break;
			case -3:
				vPrintString( "______________> Sending Right++ ");
				Print_Text(UART_WF,"R");
				break;
			case -4:
				vPrintString( "______________> Sending Right+++");
				Print_Text(UART_WF,"S");
				break;
			}

			if (x_move || y_move)
				vPrintString( "\n");
//				printf( "\ncont=%d",cont);

			x_move=0;
			y_move=0;

			cont++;

			if (cont==999999999)
				cont=4;

			for( ul = 0; ul < DELAY_LOOP_COUNT1; ul++ ){}
		} //Enabled

	}//endless while
}

void Config_Wifly ()
{
	printf( "[Config_Wifly] Initializing Wifly. \n" );

	if (Wifly_Init(UART_WF, BR_WF))
		printf( "[Config_Wifly] Init OK. \n" );

	printf( "[Config_Wifly] Conecting to WLAN... \n" );

	if ( Wifly_Connect(UART_WF, WPA2, "XXX", "YYYYYYYYYYYYYYYYY") )
		printf( "[Config_Wifly] WLAN connected. \n" );

	//Configurando Wifly UDP

	printf( "[Config_Wifly] Configurando modulo WiFly... \n" );

	Wifly_Set (UART_WF,"set ip proto 1",500);					//Enable UDP

	Wifly_Set (UART_WF,"set broadcast interval 0",500);			//Disable Broadcast


	Wifly_Set (UART_WF,"set ip host 255.255.255.255",500);		//255.255.255.255
	Wifly_Set (UART_WF,"set ip flags 0x00",500);				//Disable Autoparing

	Wifly_Set (UART_WF,"set ip remote 8888",500);
	Wifly_Set (UART_WF,"set ip local 8888",500);
	Wifly_Set (UART_WF,"set wlan join 1",500);

	Wifly_Set (UART_WF,"save",500);
	Wifly_Set (UART_WF,"reboot",500);

	//static char Buffer[GetSIZE];
}

void IO_Init()
{
	 printf( "\n[IO_Init] Initializing..." );

	 //LPC_PINCON->PINSEL1	&= ( ~( 3 << 22 ) );	//Port 0.27 Set bit 22 & 23 to 00 (GPIO)
	 //LPC_PINCON->PINSEL2	&= ( ~( 3 << 12 ) );

	 GPIOSetDir(0, 2, 	INPUT);
	 GPIOSetDir(0, 3, 	INPUT);
	 GPIOSetDir(0, 21, 	INPUT);
	 GPIOSetDir(2, 6, 	INPUT);

	 GPIOSetPull(0, 2, 	PULLDOWN);
	 GPIOSetPull(0, 3, 	PULLDOWN);
	 GPIOSetPull(0, 21, PULLDOWN);
	 GPIOSetPull(2, 6, 	PULLDOWN);
}

void ADC_Init()
{
	printf( "\n[ADC_Init] Initializing ADC: Clock=%d",ADC_CLK );
	ADCInit(ADC_CLK);

	int i=0;
	for (i = 0; i < 5000000; i++);
}

/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}


