
/*
===============================================================================
 Name        : adc.c
 Author      : Eduardo Rafael Sepulveda de Ugarte
 Version     : 1.0
 Description : Management of adc of LPCExpresso1769 for PICKMOUSE Project.
 ===============================================================================
 */

#include "lpc17xx.h"
#include "adc.h"
//#include "basic_io.h"
//#include "lpc_types.h"
//#include "PickMouse_types.h"


#ifndef _BV
#define _BV(_x_) (1UL << (_x_))
#endif

volatile uint32_t ADCValue[ADC_NUM];
volatile uint32_t ADCIntDone = 0;
volatile uint32_t BurstCounter = 0;
volatile uint32_t OverRunCounter = 0;

#if BURST_MODE
volatile uint32_t channel_flag = 0; 
#endif

#if ADC_INTERRUPT_FLAG

/******************************************************************************
** Function name:		ADC_IRQHandler
**
** Descriptions:		ADC interrupt handler
**
** parameters:			None
** Returned value:		None
** 
******************************************************************************/
void ADC_IRQHandler (void) 
{
  uint32_t regVal;
  volatile uint32_t dummy;
  int i;
  
  regVal = LPC_ADC->ADSTAT;		/* Read ADC will clear the interrupt */
  if ( regVal & 0x0000FF00 )	/* check OVERRUN error first */
  {
	OverRunCounter++;
  	for ( i = 0; i < ADC_NUM; i++ )
  	{
  	  regVal = (regVal & 0x0000FF00) >> 0x08;
  	  /* if overrun, just read ADDR to clear */
  	  /* regVal variable has been reused. */
      if ( regVal & _BV(i) )
  	  {
        //dummy = LPC_ADC->ADDR[i]; for cmsis2

        switch (i) {
			case 0:
				dummy = LPC_ADC->ADDR0;
				break;
			case 1:
				dummy = LPC_ADC->ADDR1;
				break;
			case 2:
				dummy = LPC_ADC->ADDR2;
				break;
			case 3:
				dummy = LPC_ADC->ADDR3;
				break;
			case 4:
				dummy = LPC_ADC->ADDR4;
				break;
			case 5:
				dummy = LPC_ADC->ADDR5;
				break;
			case 6:
				dummy = LPC_ADC->ADDR6;
				break;
			case 7:
				dummy = LPC_ADC->ADDR7;
				break;
		}

  	  }
  	}
	LPC_ADC->ADCR &= ~((0x7<<24)|(0x1<<16));	/* stop ADC now, turn off BURST bit. */
  	ADCIntDone = 1;
  	return;	
  }

  for ( i = 0; i < ADC_NUM; i++ )
  {
    if ( regVal & _BV(i) )
	{
    	uint32_t adcbuf = 0;
        switch (i) {
 			case 0:
 				adcbuf = LPC_ADC->ADDR0;
 				break;
 			case 1:
 				adcbuf = LPC_ADC->ADDR1;
 				break;
 			case 2:
 				adcbuf = LPC_ADC->ADDR2;
 				break;
 			case 3:
 				adcbuf = LPC_ADC->ADDR3;
 				break;
 			case 4:
 				adcbuf = LPC_ADC->ADDR4;
 				break;
 			case 5:
 				adcbuf = LPC_ADC->ADDR5;
 				break;
 			case 6:
 				adcbuf = LPC_ADC->ADDR6;
 				break;
 			case 7:
 				adcbuf = LPC_ADC->ADDR7;
 				break;
 		}
	  ADCValue[i] = ( adcbuf >> 4 ) & 0xFFF;
	}
  }

#if BURST_MODE
  BurstCounter++;
  channel_flag |= (regVal & 0xFF);
  if ( (channel_flag & 0xFF) == 0xFF )
  {
	/* All the bits in have been set, it indicates all the ADC 
	channels have been converted. */
	LPC_ADC->ADCR &= ~(0x1<<16);	/* Clear BURST mode, stop ADC now */
  	channel_flag = 0; 
	ADCIntDone = 1;
  }
#else
  LPC_ADC->CR &= ~(0x7<<24);	/* stop ADC now */ 
  ADCIntDone = 1;
#endif

  return;
}

#endif


/*****************************************************************************
** Function name:		ADCInit
**
** Descriptions:		initialize ADC channel
**
** parameters:			ADC clock rate
** Returned value:		None
** 
*****************************************************************************/
void ADCInit( uint32_t ADC_Clk )
{
	uint32_t 	i;
	uint32_t	pclkdiv;
	uint32_t	pclk;

	/* Clear data reception area. */
	for ( i = 0; i < ADC_NUM; i++ )
	{
	ADCValue[i] = 0x0;
	}

//Power
	/*Clear PDN bit 21 AD0CR */
	//LPC_ADC->ADCR = ( 1 << 21 );	//Produce Hard fault

	//Delay
	//for (i = 0; i < 5000000; i++);

//Clock
	/* Enable CLOCK into ADC controller */
	LPC_SC->PCONP |= (1 << 12); 	//Bit 12 PCADC

	/* Read value of peripheral clock divider and then use this value
	   to calculate clock frequency to ADC
	   By default, the PCLKSELx value is zero, thus, the PCLK for
	   all the peripherals is 1/4 of the SystemFrequency.

	   Bit 24~25 PCLK_ADC is for ADC page 849 Manual 4.7.3
	   LPC17xx.c line 144
	*/
	pclkdiv = (LPC_SC->PCLKSEL0 >> 24) & 0x03;
	switch ( pclkdiv )
	{
	case 0x00:
	default:
	  pclk = SystemCoreClock/4;
	break;
	case 0x01:
	  pclk = SystemCoreClock;
	break;
	case 0x02:
	  pclk = SystemCoreClock/2;
	break;
	case 0x03:
	  pclk = SystemCoreClock/8;
	break;
	}

//Pins
// ADC inputs, AD0.0~7

//-- PINSEL ----------------------------------------------
	/* PINSEL0 32 BITS FF FF FF FF
	 * A0.7 Pin 0.2 Bits 5:4 function 10
	 * A0.6 Pin 0.3 Bits 7:6 function 10
	 * Not used  pins P0.2 & P0.3 */
	//LPC_PINCON->PINSEL0 &= ~0x000000F0;	// Set to  0 bits 4,5,6,7
	//LPC_PINCON->PINSEL0 |= 0x000000A0;  // Set to  1010

	/* PINSEL0 32 BITS FF FF FF FF
	 * A0.0 PIN 0.23 Bits 14-15 function 01
	 * A0.1 PIN 0.24 Bits 16-17 function 01
	 * A0.2 PIN 0.25 Bits 18-19 function 01
	 * A0.3 PIN 0.26 Bits 14-15 function 01 */
	LPC_PINCON->PINSEL1 &= ~0x003FC000;			//Position:                     98 7654 3210
	LPC_PINCON->PINSEL1 |= 0x00154000;  		//154000 = 00 00 01 01 01 00 00 00 0000 0000

	//Not used P1.30 & P1.31
	//LPC_PINCON->PINSEL3 |= 0xF0000000;	/* P1.30~31, A0.4~5, function 11 */

//-- PINMODE ---------------------------------------------

	/* No pull-up no pull-down (function 10) on these ADC pins.
	 * Not used */
	//LPC_PINCON->PINMODE0 &= ~0x000000F0;	//Pin 2 Bits 4:5 / Pin 3 Bits 6:7
	//LPC_PINCON->PINMODE0 |= 0x000000A0;

	/* Pin has an on-chip pull-down resistor enabled (function 11)
	 * Pin 23 Bits 14:15
	 * Pin 24 Bits 16:17
	 * Pin 25 Bits 18:19
	 * Pin 26 Bits 20:21	*/
	LPC_PINCON->PINMODE1 &= ~0x000FC000;	//3FC000= 0000 1111 1100 0000 0000 0000
	LPC_PINCON->PINMODE1 |= 0x002A8000;

	//Not used
	//LPC_PINCON->PINMODE3 &= ~0xF0000000;
	//LPC_PINCON->PINMODE3 |= 0xA0000000;

	//NEW
	/* After configuring PINSEL, ADC needs a delay before starting sampling (Otherwise, higher values will be read)  */
	//for (i = 0; i < 5000000; i++);

	/* ADCR SEL bits 0..7
	 * SEL 8 bits selects which of the AD0.7:0 pins is (are) to be sampled and converted.
	 * For AD0, bit 0 selects Pin AD0.0, and bit 7 selects pin AD0.7.
	 * In software-controlled mode, only one of these bits should be 1.
	 * In hardware scan mode, any value containing 1 to 8 ones is allowed.
	 * All zeroes is equivalent to 0x01.
	*/
	//LPC_ADC->ADCR = ( 0x01 << 0 ) |  		/* SEL=1,select channel 0~7 on ADC0 (see info up*/
	LPC_ADC->ADCR = 0x07  |  				/* SEL=0000 0111 , select channels 0,1,2 on ADC0 (see info up*/
		( ( pclk  / ADC_Clk - 1 ) << 8 ) |  /* CLKDIV = Fpclk / ADC_Clk - 1 */ 
		( 0 << 16 ) | 						/* BURST = 0, no BURST, software controlled */
		( 0 << 17 ) |  						/* Reserved?? CLKS = 0, 11 clocks/10 bits */
		( 1 << 21 ) |  						/* PDN = 1, normal operation */
		( 0 << 24 ) |  						/* START = 0 A/D conversion stops */
		( 0 << 25 ) |
		( 0 << 26 ) |
		( 0 << 27 );						/* EDGE = 0 (CAP/MAT singal falling,trigger A/D conversion) */

	//Some Sleep
	for (i = 0; i < 5000000; i++);

  /* If POLLING, no need to do the following */
//INTERRUPTIONS
  #if ADC_INTERRUPT_FLAG
  	NVIC_EnableIRQ(ADC_IRQn);

	#if BURST_MODE
	  LPC_ADC->ADINTEN = 0xFF;		/* Enable all interrupts */
	#else
	  LPC_ADC->ADINTEN = 0x1FF;		/* Enable all interrupts */
	#endif

#endif

  return;
}


/*****************************************************************************
** Function name:		ADCRead
**
** Descriptions:		Read ADC channel
**
** parameters:			Channel number
** Returned value:		Value read, if interrupt driven, return channel #
** 
*****************************************************************************/
uint32_t ADCRead( uint8_t channelNum, bool filter )
{
	//printf( "\n [ADCREAD] Reading %d...",channelNum);

#if !ADC_INTERRUPT_FLAG

  uint32_t regVal = 0;
  //uint32_t regVal2 = 0;
  //uint32_t regVal3 = 0;
  //uint32_t shift1 = 0;
  //uint32_t shift2 = 0;

  uint32_t ADC_Data1 = 0;
  //uint32_t ADC_Data2 = 0;

#endif

  /* channel number is 0 through 7 */
  if ( channelNum >= ADC_NUM )
  {
	  //vPrintString( "\n [ADCREAD] Error channel number=%d.",channelNum); //only for semihosting
  }

  // switch channel,start A/D convert
  //Set to 0 the 8 first bits disable all channels.
  LPC_ADC->ADCR &= 0xFFFFFF00;

  //SET the channel to be sampled
  LPC_ADC->ADCR |= (1 << channelNum);

  //START Conversion now!
  LPC_ADC->ADCR |= (1 << 24) ;

#if !ADC_INTERRUPT_FLAG
  while ( 1 )	// wait until end of A/D convert
  {	// Read result of A/D conversion
	switch (channelNum) {
			case 0:
				regVal = LPC_ADC->ADDR0;
				break;
			case 1:
				regVal = LPC_ADC->ADDR1;
				break;
			case 2:
				regVal = LPC_ADC->ADDR2;
				break;
			case 3:
				regVal = LPC_ADC->ADDR3;
				break;
			case 4:
				regVal = LPC_ADC->ADDR4;
				break;
			case 5:
				regVal = LPC_ADC->ADDR5;
				break;
			case 6:
				regVal = LPC_ADC->ADDR6;
				break;
			case 7:
				regVal = LPC_ADC->ADDR7;
				break;
	}//switch

    //regVal2 = LPC_ADC->ADCR[channelNum];
	//regVal3 = LPC_ADC->DR[channelNum];


	if ( regVal & ADC_DONE )
	{
		//printf( "\n [ADCREAD] channel=%u value1=%u ",channelNum, regVal ); 	//only for semihosting
		//printf( "\n [ADCREAD] channel=%u value2=%u ",channelNum, regVal2 ); 	//only for semihosting
		//printf( "\n [ADCREAD] channel=%u value3=%u ",channelNum, regVal3 ); 	//only for semihosting
	  break;
	}
  }	//while
        
  LPC_ADC->ADCR &= 0xF8FFFFFF;	// Stop ADC now "START bits"

  if ( regVal & ADC_OVERRUN )	// Save data when it's not overrun, otherwise, return zero
  {
	  //printf( "\n [ADCREAD] OVERRRUN! ",channelNum, regVal );
	  return ( 0 );
  }

  if (filter)
  {
	  //shift1 = ( regVal >> 4 );
	  //shift2 = ( regVal2 >> 4 );

	  //printf( "\n [ADCREAD] channel=%u shift1=%u ",channelNum, shift1 ); //only for semihosting
	  //printf( "\n [ADCREAD] channel=%u shift2=%u ",channelNum, shift2 ); //only for semihosting

	  //ADC_Data1 = shift1 & 0xFFF;
	  //ADC_Data2 = shift2 & 0xFFF;

	  //printf( "\n [ADCREAD] channel=%u ADC_Data1=%u ",channelNum, ADC_Data1 ); //only for semihosting
	  //printf( "\n [ADCREAD] channel=%u ADC_Data2=%u ",channelNum, ADC_Data2 ); //only for semihosting

	  ADC_Data1 = ( regVal >> 4 ) & 0xFFF;
  	  return ( ADC_Data1 );	/* return A/D conversion value filtered*/
  }
  else
	  return ( regVal );	/* return A/D conversion value */

#else
  return ( channelNum );	/* if it's interrupt driven, the ADC reading is 
							done inside the handler. so, return channel number */
#endif
}

