/*
===============================================================================
 Name        : leds.c
 Author      : Eduardo Rafael Sepulveda de Ugarte
 Version     : 1.0
 Description : Management of the leds of LPCExpresso1769 for PICKMOUSE Project.
 ===============================================================================
 */

#include "leds.h"
#include "LPC17xx.h"

#define redLED_BIT 						( 22 )

void led_init()
{

	/* Initialise P0_22 for the LED. Datasheet RevB page 5.
	 * page 104 Pin Connect Block, PINSEL1 is the upper 16 bits of P0
	 * we want to set to 00 bits 12&13 as we want the primary default function of the GPIO port
	 */
	LPC_PINCON->PINSEL1	&= ( ~( 3 << 12 ) );

	/*
	 * set the direction register. set direction output (1)
	 * to the bit redLED_BIT mapped to the position (22)
	 */
	LPC_GPIO0->FIODIR |= ( 1 << redLED_BIT );
}

void led_on()
{
	LPC_GPIO0->FIOSET |= 1 << redLED_BIT;//turn it on! does nothing if it is already on.
}

void led_off()
{
	LPC_GPIO0->FIOCLR |= 1 << redLED_BIT;//turn it off! does nothing if it is already off.
}

void led_toggle()
{
	unsigned long ulLEDState;
	/* Obtain the current P0 state. */
	ulLEDState = LPC_GPIO0->FIOPIN;

	/* Turn the LED off if it was on, and on if it was off. */
	LPC_GPIO0->FIOCLR = ulLEDState & ( 1 << redLED_BIT );
	LPC_GPIO0->FIOSET = ( ( ~ulLEDState ) & ( 1 << redLED_BIT ) );
}
