/*
===============================================================================
Name        : pickmouse.cpp
Author      : Eduardo Rafael Sepulveda de Ugarte
Version     : 1.0
Description : Reception of UDP packets from PickMouse device and execure commands 
				about the movement of the pointer and mouse buttons.
===============================================================================
*/

#include "stdafx.h"

#include<stdio.h>
#include<winsock2.h>
#include <time.h>
#include <ctime>

#pragma comment(lib,"ws2_32.lib") //Winsock Library

//UDP
#define BUFLEN 512  //Max length of buffer
#define PORT 8888   //The port on which to listen for incoming data


//Mouse
#define SCREEN_WIDTH 1600
#define SCREEN_HEIGHT 900

bool pressed = false;

void MouseSetup(INPUT * buffer)
{
	buffer->type = INPUT_MOUSE;
	buffer->mi.dx = (0 * (0xFFFF / SCREEN_WIDTH));
	buffer->mi.dy = (0 * (0xFFFF / SCREEN_HEIGHT));
	buffer->mi.mouseData = 0;
	buffer->mi.dwFlags = MOUSEEVENTF_ABSOLUTE;
	buffer->mi.time = 0;
	buffer->mi.dwExtraInfo = 0;
}

void MouseMoveAbsolute(INPUT *buffer, int x, int y)
{
	buffer->mi.dx = (x * (0xFFFF / SCREEN_WIDTH));
	buffer->mi.dy = (y * (0xFFFF / SCREEN_HEIGHT));
	buffer->mi.dwFlags = (MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MOVE);

	SendInput(1, buffer, sizeof(INPUT));
}

void MouseClick(INPUT *buffer)
{
	buffer->mi.dwFlags = (MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_LEFTDOWN);
	SendInput(1, buffer, sizeof(INPUT));

	Sleep(10);

	buffer->mi.dwFlags = (MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_LEFTUP);
	SendInput(1, buffer, sizeof(INPUT));
}


void LeftClick()
{
	INPUT    Input = { 0 };
	// left down 
	Input.type = INPUT_MOUSE;
	Input.mi.dwFlags = MOUSEEVENTF_LEFTDOWN;
	::SendInput(1, &Input, sizeof(INPUT));

	// left up
	::ZeroMemory(&Input, sizeof(INPUT));
	Input.type = INPUT_MOUSE;
	Input.mi.dwFlags = MOUSEEVENTF_LEFTUP;
	::SendInput(1, &Input, sizeof(INPUT));
}
void LeftHalfClic()
{
	INPUT    Input = { 0 };

	if (pressed)
	{
		// left up
		::ZeroMemory(&Input, sizeof(INPUT));
		Input.type = INPUT_MOUSE;
		Input.mi.dwFlags = MOUSEEVENTF_LEFTUP;
		::SendInput(1, &Input, sizeof(INPUT));
		pressed = false;
	}
	else
	{
		// left down 
		Input.type = INPUT_MOUSE;
		Input.mi.dwFlags = MOUSEEVENTF_LEFTDOWN;
		::SendInput(1, &Input, sizeof(INPUT));
		pressed = true;
	}
	
}

void RightClick()
{
	INPUT    Input = { 0 };
	// right down 
	Input.type = INPUT_MOUSE;
	Input.mi.dwFlags = MOUSEEVENTF_RIGHTDOWN;
	::SendInput(1, &Input, sizeof(INPUT));

	// right up
	::ZeroMemory(&Input, sizeof(INPUT));
	Input.type = INPUT_MOUSE;
	Input.mi.dwFlags = MOUSEEVENTF_RIGHTUP;
	::SendInput(1, &Input, sizeof(INPUT));
}

int _tmain(int argc, _TCHAR* argv[])
{
	SOCKET s;
	struct sockaddr_in server, si_other;
	int slen, recv_len;
	char buf[BUFLEN];
	WSADATA wsa;
	
	time_t timerU;
	clock_t nowU =0;
	clock_t lastU =0;

	int avanceU = 5;

	slen = sizeof(si_other);

	//Initialise winsock
	printf("\nInitialising Winsock...");
	if (WSAStartup(MAKEWORD(2, 2), &wsa) != 0)
	{
		printf("Failed. Error Code : %d", WSAGetLastError());
		exit(EXIT_FAILURE);
	}
	printf("\nInitialised.");

	//Create a socket
	if ((s = socket(AF_INET, SOCK_DGRAM, 0)) == INVALID_SOCKET)
	{
		printf("\nCould not create socket : %d", WSAGetLastError());
	}
	printf("\nSocket created.");

	//Prepare the sockaddr_in structure
	server.sin_family = AF_INET;
	server.sin_addr.s_addr = INADDR_ANY;
	server.sin_port = htons(PORT);

	//Bind
	if (bind(s, (struct sockaddr *)&server, sizeof(server)) == SOCKET_ERROR)
	{
		printf("\nBind failed with error code : %d", WSAGetLastError());
		exit(EXIT_FAILURE);
	}
	puts("\nBind done");

	printf("\nWaiting for data...");
	POINT p;

	nowU = clock();
	lastU = clock();

	//keep listening for data
	while (1)
	{	
		fflush(stdout);
		//clear the buffer by filling null, it might have previously received data
		memset(buf, '\0', BUFLEN);
		
		//Obtaint current pointer position
		//GetCursorPos(&p);

		//printf("Current CURSOR position x=%ld y=%ld", p.x, p.y);

		//try to receive some data, this is a blocking call
		if ((recv_len = recvfrom(s, buf, BUFLEN, 0, (struct sockaddr *) &si_other, &slen)) == SOCKET_ERROR)
		{
			printf("\nrecvfrom() failed with error code : %d", WSAGetLastError());
			exit(EXIT_FAILURE);
		}

		//print details of the client/peer and the data received
		//printf("Received packet from %s:%d %s\n", inet_ntoa(si_other.sin_addr), ntohs(si_other.sin_port), buf);
		//printf("\nData: %s", buf);
			
		GetCursorPos(&p);
		
		if (strcmp(buf, "F") == 0)
		{
			SetCursorPos(p.x, p.y - 5);
			printf("\nUP!");
		}
		else
		if (strcmp(buf, "G") == 0)
		{
			SetCursorPos(p.x, p.y - 10);
			printf("\nUP+!");
			/*for (int i = 0; i < 2; i++)
			{
				SetCursorPos(p.x, p.y - 5);
			}*/
		}
		else
		if (strcmp(buf, "H") == 0)
		{
			SetCursorPos(p.x, p.y - 20);
			printf("\nUP++!");
			//just for testing... trying to make it soft
			/*for (int i = 0; i < 4; i++) 
			{
				SetCursorPos(p.x, p.y - 5);
			}*/
		}
		else
		if (strcmp(buf, "I") == 0)
		{
			SetCursorPos(p.x, p.y - 40);
			printf("\nUP+++!");
			/*for (int i = 0; i < 8; i++)
			{
				SetCursorPos(p.x, p.y - 5);
			}*/
		}
		else //------------------- DOWN -----------------------------------------
		if (strcmp(buf, "A") == 0)
		{
			SetCursorPos(p.x, p.y + 5);
			printf("\nDOWN!");
			
		}
		else
		if (strcmp(buf, "B") == 0)
		{
			SetCursorPos(p.x, p.y + 10);
			printf("\nDOWN+!");
			
			/*for (int i = 0; i < 2; i++)
			{
				SetCursorPos(p.x, p.y + 5);
			}*/
		}
		else 
		if (strcmp(buf, "C") == 0)
		{
			SetCursorPos(p.x, p.y + 20);
			printf("\nDOWN++!");
			/*for (int i = 0; i < 4; i++)
			{
				SetCursorPos(p.x, p.y + 5);
			}*/
		}
		else
		if (strcmp(buf, "D") == 0)
		{
			SetCursorPos(p.x, p.y + 40);
			printf("\nDOWN+++!");
			/*for (int i = 0; i < 8; i++)
			{
				SetCursorPos(p.x, p.y + 5);
			}*/
		}
		else//------------------- LEFT -----------------------------------------
		if (strcmp(buf, "K") == 0)
		{
			SetCursorPos(p.x - 5, p.y);
			printf("\nLEFT!");
		}
		else
		if (strcmp(buf, "L") == 0)
		{
			SetCursorPos(p.x - 10, p.y);
			printf("\nLEFT+!");;
		}
		else
		if (strcmp(buf, "M") == 0)
		{
			SetCursorPos(p.x - 20, p.y);
			printf("\nLEFT++!");
		}
		else
		if (strcmp(buf, "N") == 0)
		{
			SetCursorPos(p.x - 40, p.y);
			printf("\nLEFT+++!");
		}
		else  //------------------- RIGHT -----------------------------------------
		if (strcmp(buf, "P") == 0)
		{
			SetCursorPos(p.x + 5, p.y );
			printf("\nRIGHT!");
		}
		else
		if (strcmp(buf, "Q") == 0)
		{
			SetCursorPos(p.x + 10, p.y);
			printf("\nRIGHT+!");
		}
		else
		if (strcmp(buf, "R") == 0)
		{
			SetCursorPos(p.x + 20, p.y);
			printf("\nRIGHT++!");
		}
		else
		if (strcmp(buf, "S") == 0)
		{
			SetCursorPos(p.x + 40, p.y);
			printf("\nRIGHT+++!");
		}
		else
		if (strcmp(buf, "X") == 0)
		{
			LeftClick();
			printf("\nLEFTCLIC <<-");
		}
		else
		if (strcmp(buf, "Y") == 0)
		{
			RightClick();
			printf("\nRIGHTCLIC ->>");
		}
		else
		if (strcmp(buf, "Z") == 0)
		{
			LeftHalfClic();
			printf("\nLEFT HALFCLIC! <- ");

		}
		else
		{
			printf("\n??????????????????????????????????????? ");
		}

		
		/*
		//now reply the client with the same data
		if (sendto(s, buf, recv_len, 0, (struct sockaddr*) &si_other, slen) == SOCKET_ERROR)
		{
			printf("sendto() failed with error code : %d", WSAGetLastError());
			exit(EXIT_FAILURE);
		}
		*/
	}

	closesocket(s);
	WSACleanup();

	return 0;
}

